const babel = require('babel-core');
const {
    join
} = require('path');

const alias = {
    '{pro}actions': 'actions',
    '{pro}global': 'global',
    '{pro}store': 'stroe',
    '{pro}lib': 'lib',
    '{common}': 'common/',
    '{components}': 'components/',
    '/pro/lib': 'lib',
    '/pro/global': 'global'
};

const options = {
    'plugins': [
        [join(__dirname, './plugin'), {
            alias,
            extName: '.es6'
        }]
    ]
};


const code = `
define([
    'base/util',
    'regular!./hiveTreeView.html',
    'text!./hiveTreeView.css',
    'util/template/tpl',
    '{pro}store/store.js',
    '{pro}actions/globalActionsCreator.js',
    '{pro}global/util.js',
    '{components}treeList/treeList.js',
    '{components}notify/notify.js',
    '{components}popupWindow/popupWindow.js',
    '{components}contextMenu/contextMenu.js',
    '{components}loading/loading.js',
    '{pro}global/constant.js',
    './showTableWindow.js',
    'ui/mask/mask',
    'regular!./tableInfoPopup.html',
    '{components}sqlEditor/sqlEditor.js',
    '{components}tooltip/tooltip.js',
    '{components}suggestion/suggestion.js'
], function(_u, _tpl, _css, _t, _store, _creator, _util, _TreeList, _Notify, _PopupWindow, _ContextMenu, _loading, _cns, _showTableWindow) {
    _t._$parseUITemplate('<textarea name="css" id="css-box">' + _css + '</textarea>');

    var HiveTreeView = Regular.extend({
        name: 'hiveTreeView',
        template: _tpl,
        config: function(data) {
            var global = _store.getState().global;
            var mng = _store.getState().management;
            data.tableColumns = global.tableColumns;
            data.createSql = global.createSql;
            data.tableUserPrivsList = mng.tableUserPrivsList;
            data.tables = [];
            data.selectedType = null; //判断是否为流表
            data.tableName = '';
            data.initState = true; //判断是否初次进入组件
            data.isSelectTableHandled = false; //selectTable函数是否运行过
            data.tableMap = {};
            data.sources = [];
            /*
             tableItems存储了每个库的hivetblid，而tableColumns里不存在
             显示表详情必须传入hivetblid，所以组件加载时会获取一次tableItems。
             */
            data.tableItems = global.tableItems; //公用的
        },
        init: function() {
            _util.connect(this, _store, ['management', 'global']);

            if (!this.data.tableItems.length) _store.dispatch(_creator.getTablesData());

            this.$watch('keyword', function(newValue) {
                if (newValue) {
                    if (this.timer) clearTimeout(this.timer);
                    this.timer = setTimeout(function() {
                        this._getTables();
                        clearTimeout(this.timer);
                    }._$bind(this), 200);
                }
            });

            this.$watch('tableColumns', function(tableColumns) {
                this.data.sources = [];
                for (var i = 0, len = tableColumns.length; i <= len - 1; i++) {
                    this.data.sources.push({
                        name: tableColumns[i].db,
                        type: tableColumns[i].type,
                        suffix: tableColumns[i].product !== _cns.COOKIE.product ? '（所属项目：' + tableColumns[i].product + '）' : ''
                    });
                }
            });
        },
        refreshData: function(notify, onSuccess) {
            this.data.isSelectTableHandled = false;
            _store.dispatch(_creator.getTableColumns(function() {
                this.data.tableMap = {};
                if (onSuccess) onSuccess();
                if (!this.data.isSelectTableHandled) this._getTables(this.data.selectedDb);
                if (notify) _Notify.success('刷新数据表成功');
            }._$bind(this)));
            /*
             重新获取库信息，由于现在实际不能建库，而本组件只需要用到库id，
             因此理论上tableItems可以不刷新，但为了保证数据同步本处还是做了刷新，
             但没有放到上面getTableColumns的回调中，以提高响应速度。
             */
            // _store.dispatch(_creator.getTablesData());
        },
        changeDb: function(event) {
            var data = this.data,
                value = event.value;
            this._getTables(value, (data.tableName ? data.tableName : ''));
        },
        /**
         * 获取表数据
         * @param  {string} db          数据库名，只获取该库下的表
         * @param  {string} selectedTable 可选，需要高亮并打开的表名
         * @return {void}
         */
        _getTables: function(db, selectedTable) {
            setTimeout(function() {
                var tableColumns = this.data.tableColumns;
                var keyword = this.data.keyword || '';
                var tableMap = this.data.tableMap;
                var ret = [];
                var tables = [];
                this.data.db = db = db || this.data.selectedDb;
                // 生成需要选中的表id
                var selectedTableId = _util.uuid();
                if (!db) return;
                for (var i = 0, len = tableColumns.length; i <= len - 1; i++) {
                    if (db === tableColumns[i].db) {
                        this.data.selectedType = tableColumns[i].type;
                        tables = tableColumns[i].tables;
                        break;
                    }
                }
                if (!tableMap[db]) {
                    for (var j = 0, k = tables.length; j < k; j++) {
                        var table = tables[j];
                        if (!keyword || table.name.indexOf(keyword) >= 0) {
                            var tableData = {
                                name: table.name,
                                path: table.name,
                                type: table.type,
                                columns: table.columns,
                                children: [],
                                configs: [{
                                    iconClass: 'icon-more u-popupwindow',
                                    position: 'behind',
                                    onClick: this.onClickMore._$bind(this)
                                }]
                            };
                            if (selectedTable && table.name === selectedTable) {
                                // 设置表打开
                                tableData.open = true;
                                // 设置表高亮
                                this.data.selectedTable = selectedTable;
                                // 设置要选中的表项目id
                                tableData.uuid = selectedTableId;
                                tableData.children = this._getColumns(table.name, table.columns);
                            }
                            tableMap[db] || (tableMap[db] = []);
                            tableMap[db].push(tableData);
                            ret.push(tableData);
                        }
                    }
                } else {
                    ret = tableMap[db].filter(function(table) {
                        return table.name.indexOf(keyword) >= 0;
                    });
                }
                this.data.tables = ret;
                this.$update('tables', ret);
                if (selectedTable) {
                    // dom准备完毕后，移动视图到选中的元素
                    setTimeout(function() {
                        _util.scrollIntoView(selectedTableId);
                    }, 50);
                }
            }._$bind(this));
        },
        /**
         * 点击查看更多后的操作
         * @param  {object} event event对象
         * @param  {object} info  抛出的表数据对象
         * @return {void}
         */
        onClickMore: function(event, info) {
            this._showContextMenu(info.name, event.target.parentNode);
            // 停止冒泡
            event.stopPropagation();
        },
        // treelist右键事件弹出菜单
        showContextMenu: function(event) {
            var element = event.event.target;
            var injectElement;
            var tableName = '';
            if (element.className !== 'u-treelist-title') {
                injectElement = element.parentNode;
            } else {
                injectElement = element;
            }

            if (event.type === 'list') {
                tableName = event.item.name;
            } else if (event.type === 'item') {
                tableName = event.item.path.split('.')[0];
            }
            this._showContextMenu(tableName, injectElement);
        },
        // 弹出菜单项
        _showContextMenu: function(tableName, injectElement) {
            var options = {
                data: {
                    position: event,
                    clazz: 'hivetv-contextmenu',
                    items: [{
                        title: '查看表详情',
                        click: function() {
                            this.onMenuShowTable(tableName);
                        }._$bind(this)
                    }]
                }
            };
            _ContextMenu.show(options, injectElement);
        },
        _getColumns: function(table, columns) {
            if (this.data.selectedType === 'kafka') {
                for (var i = columns.length - 1; i >= 0; i--) {
                    if (columns[i].name === 'kafka_message' || columns[i].name === 'kafka_timestamp' || columns[i].name === 'kafka_position' || columns[i].name === 'kafka_consumer') {
                        columns.splice(i, 1);
                    }
                }
            }
            return columns.map(function(col) {
                var configs = [{
                    info: col.type,
                    infoClass: 'col-type',
                    position: 'behind'
                }];
                if (col.part) configs.push({
                    iconClass: 'icon-dot',
                    title: '分区字段',
                    position: 'front'
                });
                return {
                    name: col.name,
                    title: col.part ? col.name + '（分区字段）' : col.name,
                    path: table + '.' + col.name,
                    children: null,
                    configs: configs
                };
            });
        },
        toggle: function() {
            this.data.closed = !this.data.closed;
        },
        explorerSelect: function(event) {
            var item = event.item;
            if (!item.children || item.children.length) return;
            item.children = this._getColumns(item.name, item.columns);
        },
        // 手动选中某个库中的某个表
        selectTable: function(poolName, tableName, type) {
            // 刷新数据表信息
            this.refreshData(false, function() {
                // 清空关键字
                var table;
                this.data.keyword = '';
                this.data.tableName = tableName;

                //设置type
                this.data.selectedType = type;
                // 获取表信息
                this.data.isSelectTableHandled = true;
                this._getTables(poolName, tableName);
            }._$bind(this));
        }
    });

    //extend component
    HiveTreeView.implement(_u._$merge(
        _showTableWindow
    ));
});
`
const result = babel.transform(code, options);

console.log(result.code);
